body{margin:0;font-family:-apple-system,BlinkMacSystemFont,"Segoe UI","Roboto","Oxygen","Ubuntu","Cantarell","Fira Sans","Droid Sans","Helvetica Neue",sans-serif;-webkit-font-smoothing:antialiased;-moz-osx-font-smoothing:grayscale}code{font-family:source-code-pro,Menlo,Monaco,Consolas,"Courier New",monospace}.toolbar .ctrl_btn{margin-left:10px}.flow{flex-direction:column;display:flex;height:100%}.flow aside{border-right:1px solid #eee;padding:15px 10px;font-size:12px;background:#fcfcfc}.flow aside>*{margin-bottom:10px;cursor:grab}.flow .reactflow-wrapper{flex-grow:1;height:100%}@media screen and (min-width:768px){.flow{flex-direction:row}.flow aside{width:20%;max-width:180px}}.outputkey{margin-left:auto;margin-right:auto;width:8em}.selectFilePath{max-width:75%}.algoName,.selectFilePath{text-underline-offset:3px;word-break:break-all;display:inline-block;overflow:hidden;text-overflow:ellipsis;white-space:nowrap}.algoName{max-width:85%}.flowbutton{width:25px;height:25px;background:#eee;border:1px solid #fff;cursor:pointer;border-radius:50%;font-size:20px;line-height:1}.flowbutton:hover{box-shadow:0 0 6px 2px rgba(0,0,0,.08)}.flowbutton-foreignobject body{background:transparent;width:50px;height:50px;display:flex;justify-content:center;align-items:center;min-height:40px}
/*# sourceMappingURL=main.08a92549.chunk.css.map */