# from optinist.wrappers.optinist_wrapper.basic_neural_analysis.cell_grouping import (
#     cell_grouping,
# )
from optinist.wrappers.optinist_wrapper.basic_neural_analysis.eta import ETA

basic_neural_analysis_wrapper_dict = {
    "eta": {"function": ETA},
    # 'cell_grouping': {
    #     'function': cell_grouping
    # }
}
